from __future__ import annotations

import pytest

from pipdeptree._cli import build_parser, get_options


def test_parser_default() -> None:
    parser = build_parser()
    args = parser.parse_args([])
    assert not args.json
    assert args.output_format is None


def test_parser_j() -> None:
    parser = build_parser()
    args = parser.parse_args(["-j"])
    assert args.json
    assert args.output_format is None


def test_parser_json() -> None:
    parser = build_parser()
    args = parser.parse_args(["--json"])
    assert args.json
    assert args.output_format is None


def test_parser_json_tree() -> None:
    parser = build_parser()
    args = parser.parse_args(["--json-tree"])
    assert args.json_tree
    assert not args.json
    assert args.output_format is None


def test_parser_mermaid() -> None:
    parser = build_parser()
    args = parser.parse_args(["--mermaid"])
    assert args.mermaid
    assert not args.json
    assert args.output_format is None


def test_parser_pdf() -> None:
    parser = build_parser()
    args = parser.parse_args(["--graph-output", "pdf"])
    assert args.output_format == "pdf"
    assert not args.json


def test_parser_svg() -> None:
    parser = build_parser()
    args = parser.parse_args(["--graph-output", "svg"])
    assert args.output_format == "svg"
    assert not args.json


@pytest.mark.parametrize(
    ("should_be_error", "depth_arg", "expected_value"),
    [
        (True, ["-d", "-1"], None),
        (True, ["--depth", "string"], None),
        (False, ["-d", "0"], 0),
        (False, ["--depth", "8"], 8),
        (False, [], float("inf")),
    ],
)
def test_parser_depth(should_be_error: bool, depth_arg: list[str], expected_value: None | float) -> None:
    parser = build_parser()

    if should_be_error:
        with pytest.raises(SystemExit):
            parser.parse_args(depth_arg)
    else:
        args = parser.parse_args(depth_arg)
        assert args.depth == expected_value


@pytest.mark.parametrize(
    "args",
    [
        pytest.param(["--exclude", "py", "--all"], id="exclude-all"),
        pytest.param(["-e", "py", "--packages", "py2"], id="exclude-packages"),
        pytest.param(["-e", "py", "-p", "py2", "-a"], id="exclude-packages-all"),
    ],
)
def test_parser_get_options_exclude_combine_not_supported(args: list[str], capsys: pytest.CaptureFixture[str]) -> None:
    with pytest.raises(SystemExit, match="2"):
        get_options(args)

    out, err = capsys.readouterr()
    assert not out
    assert "cannot use --exclude with --packages or --all" in err


def test_parser_get_options_exclude_only() -> None:
    parsed_args = get_options(["--exclude", "py"])
    assert parsed_args.exclude == "py"


def test_parser_get_options_license_and_freeze_together_not_supported(capsys: pytest.CaptureFixture[str]) -> None:
    with pytest.raises(SystemExit, match="2"):
        get_options(["--license", "--freeze"])

    out, err = capsys.readouterr()
    assert not out
    assert "cannot use --license with --freeze" in err



from __future__ import annotations

import os
import sys
from inspect import getsourcefile
from pathlib import Path
from shutil import copytree
from subprocess import call  # noqa: S404
from tempfile import TemporaryDirectory
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._cli import Options


def handle_non_host_target(args: Options) -> int | None:
    # if target is not current python re-invoke it under the actual host
    py_path = Path(args.python).absolute()
    if py_path != Path(sys.executable).absolute():
        # there's no way to guarantee that graphviz is available, so refuse
        if args.output_format:
            print(  # noqa: T201
                "graphviz functionality is not supported when querying non-host python",
                file=sys.stderr,
            )
            raise SystemExit(1)
        argv = sys.argv[1:]  # remove current python executable
        for py_at, value in enumerate(argv):
            if value == "--python":
                del argv[py_at]
                del argv[py_at]
            elif value.startswith("--python"):
                del argv[py_at]

        src = getsourcefile(sys.modules[__name__])
        assert src is not None
        our_root = Path(src).parent

        with TemporaryDirectory() as project:
            dest = Path(project)
            copytree(our_root, dest / "pipdeptree")
            # invoke from an empty folder to avoid cwd altering sys.path
            env = os.environ.copy()
            env["PYTHONPATH"] = project
            cmd = [str(py_path), "-m", "pipdeptree", *argv]
            return call(cmd, cwd=project, env=env)  # noqa: S603
    return None


__all__ = [
    "handle_non_host_target",
]
